@@ -411,7 +411,7 @@ namespace LightGBM {
         cnt_in_bin.resize(num_bin_, 0);
         int i_bin = 0;
         for (int i = 0; i < num_distinct_values; ++i) {
-          if (distinct_values[i] > bin_upper_bound_[i_bin]) {
+          while (distinct_values[i] > bin_upper_bound_[i_bin] && i_bin < num_bin_ - 1) {
             ++i_bin;
           }
           cnt_in_bin[i_bin] += counts[i];
@@ -519,21 +519,6 @@ namespace LightGBM {
     }
   }
 
-
-  int BinMapper::SizeForSpecificBin(int bin) {
-    int size = 0;
-    size += static_cast<int>(VirtualFileWriter::AlignedSize(sizeof(int)));
-    size +=
-        static_cast<int>(VirtualFileWriter::AlignedSize(sizeof(MissingType)));
-    size += static_cast<int>(VirtualFileWriter::AlignedSize(sizeof(bool)));
-    size += sizeof(double);
-    size += static_cast<int>(VirtualFileWriter::AlignedSize(sizeof(BinType)));
-    size += 2 * sizeof(double);
-    size += bin * sizeof(double);
-    size += static_cast<int>(VirtualFileWriter::AlignedSize(sizeof(uint32_t))) * 2;
-    return size;
-  }
-
   void BinMapper::CopyTo(char * buffer) const {
     std::memcpy(buffer, &num_bin_, sizeof(num_bin_));
     buffer += VirtualFileWriter::AlignedSize(sizeof(num_bin_));
@@ -661,26 +646,35 @@ namespace LightGBM {
     }
   }
 
-  MultiValBin* MultiValBin::CreateMultiValBin(data_size_t num_data, int num_bin, int num_feature, double sparse_rate) {
-    const double multi_val_bin_sparse_threshold = 0.25f;
+  MultiValBin* MultiValBin::CreateMultiValBin(data_size_t num_data, int num_bin, int num_feature,
+    double sparse_rate, const std::vector<uint32_t>& offsets) {
     if (sparse_rate >= multi_val_bin_sparse_threshold) {
       const double average_element_per_row = (1.0 - sparse_rate) * num_feature;
       return CreateMultiValSparseBin(num_data, num_bin,
                                      average_element_per_row);
     } else {
-      return CreateMultiValDenseBin(num_data, num_bin, num_feature);
+      return CreateMultiValDenseBin(num_data, num_bin, num_feature, offsets);
     }
   }
 
   MultiValBin* MultiValBin::CreateMultiValDenseBin(data_size_t num_data,
                                                    int num_bin,
-                                                   int num_feature) {
-    if (num_bin <= 256) {
-      return new MultiValDenseBin<uint8_t>(num_data, num_bin, num_feature);
-    } else if (num_bin <= 65536) {
-      return new MultiValDenseBin<uint16_t>(num_data, num_bin, num_feature);
+                                                   int num_feature,
+                                                   const std::vector<uint32_t>& offsets) {
+    // calculate max bin of all features to select the int type in MultiValDenseBin
+    int max_bin = 0;
+    for (int i = 0; i < static_cast<int>(offsets.size()) - 1; ++i) {
+      int feature_bin = offsets[i + 1] - offsets[i];
+      if (feature_bin > max_bin) {
+        max_bin = feature_bin;
+      }
+    }
+    if (max_bin <= 256) {
+      return new MultiValDenseBin<uint8_t>(num_data, num_bin, num_feature, offsets);
+    } else if (max_bin <= 65536) {
+      return new MultiValDenseBin<uint16_t>(num_data, num_bin, num_feature, offsets);
     } else {
-      return new MultiValDenseBin<uint32_t>(num_data, num_bin, num_feature);
+      return new MultiValDenseBin<uint32_t>(num_data, num_bin, num_feature, offsets);
     }
   }
 
